#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#estimation of odds ratios
ORME<-function(fT,or_T,fE,or_E,fM,no_LD,LD,LD_measure,P0,Kp,int,mode_inh,r_path){
   
    #Estimation of a range of LD values which fit to fT and fM
    LD_path<-paste(r_path,"LD.R",sep="")
    source(LD_path)
    
    #Estimation of the baseline risk of disease in non-exposed individuals who do not carry the marker allele associated with the disease allele. 
    P1_int_path<-paste(r_path,"P1_int.R",sep="")
    source(P1_int_path)
    
    #Estimation of the baseline risk of disease in individuals who do not carry the marker allele. 
    P1_mainG_path<-paste(r_path,"P1_mainG.R",sep="")
    source(P1_mainG_path)
        
    ft<-1-fT
    fm<-1-fM
    fe<-1-fE
    
    #Estimation of r2/D. 
    if(LD_measure=="D"){
        d_TM<-LD
        r2<-(d_TM^2)/(fM*fm*fT*ft)
    }else if(LD_measure=="r2"){
        r2<-LD
        d_TM<-sqrt(r2*fT*ft*fM*fm)
    }
    
    #Estimation of the OR at marker locus (formula by Zondervan & Cardon (Nature Reviews, 2004, 5:89-100) and Ackerman et al. (Genome Biology, 2003, 4,4,R24)). 
    or_M<-1+(d_TM*(or_T-1))/(((fT*fm-d_TM)*(or_T-1)+fm)*fM)
    
    #Estimation of the true interaction OR. 
    or_TE<-or_T*or_E*int
        
    #Joint distribution for disease and marker locus in the general (control) population, depending on the linkage disequilibrium paramter between alleles T and M. 
    fTM<-d_TM+(fT*fM)
    ftm<-d_TM+(ft*fm)
    fTm<--d_TM+(fT*fm)
    ftM<--d_TM+(ft*fM)
    
    if(any(fTM>min(fT+10^(-10),fM+10^(-10)))){
        control_ld<-LD(fT,fM)
        if(LD_measure=="r2"){
            r2_min<-control_ld[3]
            r2_max<-control_ld[4]
            stop("Invalid LD value (r2=",r2,") given these marker and disease allele frequencies (fM=",fM,", fT=",fT,")!\n r2 should be in the range ",r2_min," to ",r2_max,"! ")
        }else{
            d_TM_min<-control_ld[1]
            d_TM_max<-control_ld[2]
            stop("Invalid LD value (D=",d_TM,") given these marker and disease allele frequencies (fM=",fM,", fT=",fT,")!\n D should be in the range ",d_TM_min," to ",d_TM_max,"! ")
        }
    }
    
    #Joint frequencies of exposure, disease locus, and marker locus in the general (control) population. 
    ftme<-fe*ftm
    fTMe<-fe*fTM
    fTme<-fe*fTm
    ftMe<-fe*ftM
    ftmE<-fE*ftm
    fTME<-fE*fTM
    fTmE<-fE*fTm
    ftME<-fE*ftM
    
    #Non-normalized joint frequencies of exposure, disease locus, and marker locus among cases. 
    gtme<-ftme
    gtMe<-ftMe
    
    gTMe<-or_T*fTMe
    gTme<-or_T*fTme
    gtmE<-or_E*ftmE
    gtME<-or_E*ftME
    
    gTmE<-or_TE*fTmE
    gTME<-or_TE*fTME
        
    sum_g<-gtme+gTMe+gTme+gtMe+gtmE+gTME+gTmE+gtME
    
    #Normalized joint frequencies of exposure, disease locus, and marker locus among cases. 
    gtme<-gtme/sum_g
    gTMe<-gTMe/sum_g
    gTme<-gTme/sum_g
    gtMe<-gtMe/sum_g
    gtmE<-gtmE/sum_g
    gTME<-gTME/sum_g
    gTmE<-gTmE/sum_g
    gtME<-gtME/sum_g
    
    #Joint frequencies of exposure and marker locus in the general (control) population. 
    fme<-ftme+fTme
    fmE<-ftmE+fTmE
    fMe<-ftMe+fTMe
    fME<-ftME+fTME
    
    fte<-ftme+ftMe
    ftE<-ftmE+ftME
    fTe<-fTme+fTMe
    fTE<-fTmE+fTME
    
    fe<-fte+fTe
    fE<-ftE+fTE
    
    ft<-fte+ftE
    fT<-fTe+fTE
    
    fm<-fme+fmE
    fM<-fMe+fME
    
    #Joint frequencies of exposure and marker locus among cases. 
    gme<-gtme+gTme
    gMe<-gtMe+gTMe
    gmE<-gtmE+gTmE
    gME<-gtME+gTME
    
    #Joint frequencies of exposure and disease locus among cases. 
    gte<-gtme+gtMe
    gTe<-gTme+gTMe
    gtE<-gtmE+gtME
    gTE<-gTmE+gTME
    
    ge<-gte+gTe
    gE<-gtE+gTE
    
    gt<-gte+gtE
    gT<-gTe+gTE
    
    gm<-gme+gmE
    gM<-gMe+gME
    
    #Estimation of odds ratios for disease locus and exposure. 
    
    or_Et<-(fte*gtE)/(ftE*gte)
    or_eT<-(fte*gTe)/(fTe*gte)
    or_ET<-(fte*gTE)/(fTE*gte)
    
    #Estimation of odds ratios for marker locus and exposure. 
    
    or_Em<-(fme*gmE)/(fmE*gme)
    or_eM<-(fme*gMe)/(fMe*gme)
    or_EM<-(fme*gME)/(fME*gme)

    #Estimation of the baseline risk of disease in unexposed non-carriers of the marker allele associated with the disease allele. 
    probD_me<-P1_int(or_eM, or_Em, or_EM, fM, fm, fE, fe, P0, Kp, no_LD,mode_inh,r_path)
    
    #Estimation of the baseline risk in unexposed individuals who do not carry the disease allele using a Newton-type algorithm (nlm()). 
    #The function to be minimized (fct()) originates from equation (3) in Gauderman, Statistics in Medicine, 2002, 21:35-50. 
    probD_m<-P1_mainG(or_M,fM,no_LD,probD_me,Kp,mode_inh)
        
    output<-matrix(NA, no_LD, 12)
    output[, 1]<-fE
    output[, 2]<-fT
    output[, 3]<-fM
    output[, 4]<-d_TM
    output[, 5]<-r2
    output[, 6]<-or_Et
    output[, 7]<-or_eT
    output[, 8]<-or_ET
    output[, 9]<-or_eM
    output[, 10]<-or_EM
    output[, 11]<-probD_m
    output[, 12]<-probD_me
      
    return(output)
}
